/*
  Name: tools.cpp
  Copyright: Polecatsoft 2005
  Author: Master Lenman
  Date: 11.08.05 14:44
  Description: Contains useful macro functions,
  that can be used in all other files.
  Game: SH '05 template
*/

#include "StdAfx.h"
#include <string>
#include <fstream>
#include <sstream>
using namespace std;

//Returns the file size of the file filename
//Can only be used for files smaller than 4GB
unsigned int fileSize(string filename)
{
	ifstream file(filename.c_str());
	if(file.fail())
		return 0;
	file.seekg(0, ios::end);
	return file.tellg();
}

//Returns false if the file filename can not be accessed (this normally means,
//that it doesn't exists)
bool fileExists(string filename)
{
	ifstream testStream(filename.c_str());
	return !testStream.fail();
}

//Below are functions of the keyStatesClass class

//Global instance of the keyStatesClass
keyStatesClass keyStates;

//Default constructor
keyStatesClass::keyStatesClass()
{
	resetAll();
}

//The function used to access the key value id
int keyStatesClass::operator [](int id)
{
	return getKey(id);
}

//Returns the value of a key
int keyStatesClass::getKey(int id)
{
	if(id >= KEY_MAX || id < 0)
		return 0;
	if(key[id] && !pressedKeys[id])
	{
		pressedKeys[id] = true;
		return 1;
	}
	if(!key[id])
		pressedKeys[id] = false;
	return 0;
}

//Returns true if ANY key was pressed
bool keyStatesClass::anyKey()
{
	bool retVal = false;
	for(int i = 0; i < KEY_MAX; ++i)
	{
		if(getKey(i))
			retVal = true;
	}
	return retVal;
}

//Sets all array values to false
void keyStatesClass::resetAll()
{
	for(int i = 0; i < KEY_MAX; ++i)
		pressedKeys[i] = false;
}

//Assign a value to the key id
void keyStatesClass::setState(int id, int value)
{
	pressedKeys[id] = value;
}

//Below are functions of the sound class

//different states the active variable can take
enum activeStates
{
	INACTIVE = 0,
	SAMPLEACTIVE = 1,
	MIDIACTIVE = 2,
	
#ifdef USE_MP3_STUFF
	MP3ACTIVE = 3,
#endif
};

//Default constructor
sound::sound()
{
	sample = NULL;
	midi = NULL;

#ifdef USE_MP3_STUFF
	mp3 = NULL;
	mp3Data = NULL;
	autoPoll = false;
#endif
	active = INACTIVE;
	paused = false;
}

//Destructor, calls destroy()
sound::~sound()
{
	destroy();
}

//Loads the file filename and turns autopoll on for mp3s if doPoll is true
bool sound::load(string filename, bool doPoll)
{
	destroy();
	//Load wav file
	if(
		filename[filename.length() - 3] == 'w' &&
		filename[filename.length() - 2] == 'a' &&
		filename[filename.length() - 1] == 'v')
	{
		sample = load_sample(filename.c_str());
		if(!sample)
		    return false;

		active = SAMPLEACTIVE;
	}
#ifdef USE_MP3_STUFF
	//Load mp3
	else if(
		filename[filename.length() - 3] == 'm' &&
		filename[filename.length() - 2] == 'p' &&
		filename[filename.length() - 1] == '3')
	{
		int length = file_size(filename.c_str());
		if(!length)
		    return false;
		mp3Data = new char[length];

		PACKFILE *mp3readfile = pack_fopen(filename.c_str(), "r");
		pack_fread(mp3Data, length, mp3readfile);
		pack_fclose(mp3readfile);

		mp3 = almp3_create_mp3(mp3Data, length);
		if(!mp3)
			return false;
		if(doPoll)
		{
			almp3_start_autopoll_mp3(mp3, 100);
			autoPoll = true;
		}
		else
		{
			autoPoll = false;
		}
		active = MP3ACTIVE;
	}
#endif
	//Load midi
	else if(
		(
			filename[filename.length() - 3] == 'm' &&
			filename[filename.length() - 2] == 'i' &&
			filename[filename.length() - 1] == 'd'
		)
		||
		(
			filename[filename.length() - 4] == 'm' &&
			filename[filename.length() - 3] == 'i' &&
			filename[filename.length() - 2] == 'd' &&
			filename[filename.length() - 1] == 'i'
		))
	{
		midi = load_midi(filename.c_str());
		if(!midi)
		    return false;

		active = MIDIACTIVE;
	}
	else
	    return false;
	return true;
}

//Plays the sound with the given parameters, returns false on error
void sound::play(int loop, int vol, int pan, int freq)
{
	if(active == MIDIACTIVE)
	{
		play_midi(midi, loop);
	}
	else if(active == SAMPLEACTIVE)
	{
		play_sample(sample, vol, pan, freq, loop);
    }
#ifdef USE_MP3_STUF
	else if(active == MP3ACTIVE)
	{
		almp3_play_ex_mp3(mp3, 24576, vol, pan, freq, loop);
	}
#endif
}

//Stops to play a sound
void sound::stop()
{
	if(active == SAMPLEACTIVE)
	{
		stop_sample(sample);
	}
	else if(active == MIDIACTIVE)
	{
		stop_midi();
	}
#ifdef USE_MP3_STUFF
	else if(active == MP3ACTIVE)
	{
		almp3_stop_mp3(mp3);
		almp3_rewind_mp3(mp3);
	}
#endif
}

//Alters the playing method of a sound
void sound::adjust(int loop, int vol, int pan, int freq)
{
	if(active == SAMPLEACTIVE)
	{
		adjust_sample(sample, vol, pan, freq, loop);
	}
#ifdef USE_MP3_STUFF
	else if(active == MP3ACTIVE)
	{
        almp3_adjust_mp3(mp3, vol, pan, freq, loop);
	}
#endif
}

//Frees memory allocated and stops playing
void sound::destroy()
{
	if(sample)
	{
		destroy_sample(sample);
		sample = NULL;
	}
	if(midi)
	{
		destroy_midi(midi);
		midi = NULL;
    }
#ifdef USE_MP3_STUFF
	if(mp3)
	{
		if(autoPoll)
			almp3_stop_autopoll_mp3(mp3);
		autoPoll = false;

		almp3_destroy_mp3(mp3);
		mp3 = NULL;
		delete [] mp3Data;
		mp3Data = NULL;
	}
#endif
	active = INACTIVE;
}

//Is there a sound currently loaded?
int sound::isActive()
{
	return active;
}

//Poll function for mp3 files
void sound::poll()
{
#ifdef USE_MP3_STUFF
	if(active == MP3ACTIVE && !autoPoll)
	{
		almp3_poll_mp3(mp3);
	}
#endif
}

BITMAP *loadBitmap(string filename)
{
	BITMAP *retVal = load_bitmap(filename.c_str(), 0);
	if(!retVal)
		error::loadBitError(filename);
	return retVal;
}

FONT *loadBitmapFont(string filename)
{
	FONT *retVal = load_bitmap_font(filename.c_str(), 0, 0);
	if(!retVal)
	    error::loadBitError(filename);
	return retVal;
}

BITMAP *createBitmap(int width, int height)
{
	BITMAP *retVal = create_bitmap(width, height);
	if(!retVal)
		error::createError();
	return retVal;
}

//Shows a file selection dialogue.
//Stores a path to the selected file in path
//w and h determine the size of the dialogue
//message will be displayed on top of the dialogue
//The files will be filtered using ext, see the allegro docs for details
//Returns false if the user closed using the cancel button
bool fileSelect(const std::string message, std::string &path,
	const std::string ext, const int w, const int h)
{
	char pathBuffer[1024];
	strncpy(pathBuffer, path.c_str(), 1024);
	int retVal = file_select_ex(message.c_str(), pathBuffer, ext.c_str(), 1024,
		w, h);
	if(!retVal)
		return false;
	path = pathBuffer;
	return true;
}

string i2str(int i, int minLength)
{
	ostringstream oss; // create a stream
	oss << i; // insert value to stream
	if((minLength -= oss.str().length()) <= 0)
		return oss.str(); // extract value and return it
	string retVal;
	while(retVal.length() < static_cast<unsigned int>(minLength))
		retVal.append(1, '0');
	return retVal + oss.str();
}

string i2str(int i)
{
	ostringstream oss; // create a stream
	oss << i; // insert value to stream
	return oss.str(); // extract value and return it
}

string f2str(float i)
{
	ostringstream oss; // create a stream
	oss << i; // insert value to stream
	return oss.str(); // extract value and return it
}

//Turns a string into an integer, returns defaultValue if the string is no int value
int str2i(const std::string str, const int defaultValue)
{
	istringstream oss(str);
	int r;
	if(!(oss >> r))
		r = defaultValue;
	return r;
}

//Turns a string into an floating point number, returns defaultValue if the string is no
//floating point value
float str2f(const std::string str, const float defaultValue)
{
	istringstream oss(str);
	float r;
	if(!(oss >> r))
		r = defaultValue;
	return r;
}
